@@ -6,20 +6,18 @@ from numpy.ctypeslib import ndpointer
 import os
 import multiprocessing as mp
 import time
-from numpy.random import default_rng
+from numpy.random import default_rng, Generator, SFC64
 import platform
+from .extensions import csimOU as _csimOU
+
 
-ST = time.time()
 
 class Result():
     def __init__(self):
         self.val = _pd.DataFrame()
 
     def update_result(self, val):
-        print(time.time() - ST)
         self.val = _pd.concat([self.val, val], axis=1) # append by columns
-        print(time.time() - ST)
-
 
 
 def simGBM(s0=10, mu=0, sigma=0.2, r=0, T=1, dt=1 / 252, sims=1000, eps=None):
@@ -101,7 +99,7 @@ def _import_csimOU():
     return fun
 
 
-def simOU(s0=5, mu=4, theta=2, sigma=1, T=1, dt=1 / 252, sims=1000, eps=None, c=False, seed=None, log_price=False):
+def simOU(s0=5, mu=4, theta=2, sigma=1, T=1, dt=1 / 252, sims=1000, eps=None, seed=None, log_price=False, c=True):
     """
     Function for calculating an Ornstein-Uhlenbeck Mean Reversion stochastic process (random walk) with multiple
     simulations
@@ -121,7 +119,8 @@ def simOU(s0=5, mu=4, theta=2, sigma=1, T=1, dt=1 / 252, sims=1000, eps=None, c=
         Starting value for mean reverting random walk at time = 0
     mu : float, int or pandas Series
         Mean that the function will revert to. Can be either a scalar value (i.e. 5) or a pandas series for a
-        time dependent mean. If array-like, it must be the same length as T/dt (i.e. the number of periods)
+        time dependent mean. If array-like, it must be the same length as T/dt (i.e. the number of periods,
+        excluding start value)
     theta : float
         Mean reversion rate, higher number means it will revert slower
     sigma : float
@@ -132,18 +131,18 @@ def simOU(s0=5, mu=4, theta=2, sigma=1, T=1, dt=1 / 252, sims=1000, eps=None, c=
         Time step size in fractions of a year. So a day would be 1/252, where 252 is the number of business
         days in a year
     sims : int
-        Number of simulations to run. By default, this is 1000.
+        Number of simulations to run. By default, this is 1000. Not used if eps is provided.
     eps : matrix-like[float]
         Random numbers to use for the returns. If provided, mu, sigma, T, dt and sims are ignored.
         Must of size (p x sims) where p is the number of periods in T, i.e. int(T/dt). 
         Excludes time 0.
-    c : bool
-        Whether or not to run C optimized code. By default False. Otherwise price python loop.
     seed : int
         To pass to numpy random number generator as seed. For testing only.
     log_price : bool
         Adds adjustment term to the mean reversion term if the prices passed are log prices. By
         default False.
+    c : bool
+        Whether or not to run C optimized code. By default True. Otherwise use python loop.
 
     Returns
     -------
@@ -154,13 +153,31 @@ def simOU(s0=5, mu=4, theta=2, sigma=1, T=1, dt=1 / 252, sims=1000, eps=None, c=
     >>> import risktools as rt
     >>> rt.simOU()
     """
+    if eps is not None:
+        sims = eps.shape[1]
 
     # number of business days in a year
     bdays_in_year = 252
+    
+    # calc periods
+    N = int(T/dt)
 
     # print half-life of theta
     print("Half-life of theta in days = ", _np.log(2) / theta * bdays_in_year)
 
+    # make mu an array of same size as N+1
+    try:
+        iter(mu)
+        if len(mu) == N:
+            mu = _np.append(mu[0] , mu)
+        else: 
+            raise ValueError("if mu is passed as an iterable, it must be of length int(T/dt)")
+    except:
+        mu = _np.ones(N+1)*mu
+        
+    # make same size as 1D array of all periods and sims
+    mu = _np.tile(_np.array(mu), sims)
+
     if c == True:
         return _simOUc(s0=s0, mu=mu, theta=theta, T=T, dt=dt, sigma=sigma, sims=sims, eps=eps, seed=seed, log_price=log_price)
     else:
@@ -168,8 +185,6 @@ def simOU(s0=5, mu=4, theta=2, sigma=1, T=1, dt=1 / 252, sims=1000, eps=None, c=
 
 
 def _simOUc(s0, theta, mu, dt, sigma, T, sims=10, eps=None, seed=None, log_price=False):
-    fun = _import_csimOU()
-
     # calc periods
     N = int(T/dt)
     
@@ -185,24 +200,14 @@ def _simOUc(s0, theta, mu, dt, sigma, T, sims=10, eps=None, seed=None, log_price
             '''
             )
     
-    # make mu an array of same size as N+1
-    try:
-        iter(mu)
-        if len(mu) != N+1:
-            raise ValueError("if mu is passed as an iterable, it must be of length int(T/dt) + 1 to account for starting value s0")
-    except:
-        mu = _np.ones((N+1))*mu
-    
-    # make same size as 1D array of all periods and sims
-    mu = _np.tile(_np.array(mu), sims)
-
     # generate a 1D array of random numbers that is based on a 
     # 2D array of size P x S where P is the number of time steps
     # including s0 (so N + 1) and S is the number of sims. 1D 
     # array will be of size P * S. This is actually the slowest
     # part.
     if eps is None:
-        rng = default_rng(seed)
+        # rng = default_rng(seed)
+        rng = Generator(SFC64(seed))
         x = rng.normal(loc=0, scale=1, size=((N+1)*sims))
         x[0] = s0
         x[::(N+1)] = s0
@@ -211,22 +216,17 @@ def _simOUc(s0, theta, mu, dt, sigma, T, sims=10, eps=None, seed=None, log_price
         x = _np.c_[_np.ones(sims)*s0, x]
         x = x.reshape((N+1)*sims)
     
-    # run simulation directly in-place on memory to save time.
-    fun(x, theta, mu, dt, sigma, sims, N+1, int(log_price))
+    x = _csimOU(x, theta, mu, dt, sigma, sims, N+1, int(log_price))
     
     return _pd.DataFrame(x.reshape((sims, N+1)).T)
 
 
 def _simOUpy(s0, mu, theta, sigma, T, dt, sims=1000, eps=None, seed=None, log_price=False):
-
+    mu = _pd.Series(mu)
+    
     # number of periods dt in T
     periods = int(T / dt)
 
-    if isinstance(mu, list):
-        assert len(mu) == (
-            periods
-        ), "Time dependent mu used, but the length of mu is not equal to the number of periods calculated."
-
     # init df with zeros, rows are steps forward in time, columns are simulations
     out = _np.zeros((periods + 1, sims))
     out = _pd.DataFrame(data=out)
@@ -234,12 +234,10 @@ def _simOUpy(s0, mu, theta, sigma, T, dt, sims=1000, eps=None, seed=None, log_pr
     # set first row as starting value of sim
     out.loc[0, :] = s0
 
-    if isinstance(mu, list):
-        mu = _pd.Series(mu)
-
     # calc gaussian vector
     if eps is None:
-        rng = default_rng(seed)
+        # rng = default_rng(seed)
+        rng = Generator(SFC64(seed))
         eps = rng.normal(size=(periods, sims))
 
     out.iloc[1:,:] = eps
@@ -252,18 +250,13 @@ def _simOUpy(s0, mu, theta, sigma, T, dt, sims=1000, eps=None, seed=None, log_pr
             continue  # skip first row
 
         # calc step
-        if isinstance(mu, list) | isinstance(mu, _pd.Series):
-            out.iloc[i, :] = (
-                out.iloc[i - 1, :]
-                + (theta * (mu.iloc[i - 1] - out.iloc[i - 1, :]) - ss) * dt
-                + sigma * out.iloc[i, :] * _np.sqrt(dt)
-            )
-        else:
-            out.iloc[i, :] = (
-                out.iloc[i - 1, :]
-                + (theta * (mu - out.iloc[i - 1, :]) - ss)* dt
-                + sigma * out.iloc[i, :] * _np.sqrt(dt)
-            )
+
+        out.iloc[i, :] = (
+            out.iloc[i - 1, :]
+            + (theta * (mu.iloc[i] - out.iloc[i - 1, :]) - ss) * dt
+            + sigma * out.iloc[i, :] * _np.sqrt(dt)
+        )
+
 
     return out
 
